// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PeriodicMap.h */

#ifndef AMDIS_PERIODICMAP_H
#define AMDIS_PERIODICMAP_H

#include "Global.h"

#include <map>

namespace AMDiS {

  class PeriodicMap
  {
  public:
    void setEntry(DegreeOfFreedom key, DegreeOfFreedom entry) {
      // no trivial entries!
      if (key == entry) 
	return;

      // if a key equal to entry exists ...
      if (getEntry(entry) >= 0) {
	if (getEntry(entry) == key) {
	  return;
	}
	// ... let it be equal entries
	setEntry(key, getEntry(entry));
	return;
      }

      // replace entries equal to key
      std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it;
      for (it = periodicMap.begin(); it != periodicMap.end(); ++it) {
	if (it->second == key) {
	  it->second = entry;
	}
      }
    
      // if key exists already
      if (getEntry(key) >= 0) {
	// insert new entry with old entry as key
	setEntry(getEntry(key), entry);
      }

      // set entry
      periodicMap[key] = entry;
    };

    DegreeOfFreedom getEntry(DegreeOfFreedom key) {
      std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it;
      it = periodicMap.find(key);
      if (it != periodicMap.end()) {
	return it->second;
      } else {
	return -1;
      }
    };

  

  protected:
    std::map<DegreeOfFreedom, DegreeOfFreedom> periodicMap;
  };

}

#endif
