// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file SurfaceQuadrature.h */

#ifndef AMDIS_SURFACEQAUDRATURE_H
#define AMDIS_SURFACEQAUDRATURE_H

#include <list>
#include "FixVec.h"
#include "Quadrature.h"

namespace AMDiS {

  /** 
   * \ingroup Integration
   *
   * \brief
   * Quadrature for element surfaces. Uses the functionality of the standard
   * Quadrature class for dim-1 but calculates new barycentric coordinates
   * for the element sides.
   */
  class SurfaceQuadrature : public Quadrature
  {
  public:
    /// Constructs a SurfaceQuadrature based on a standard Quadrature of dim-1.
    SurfaceQuadrature(Quadrature *quad, VectorOfFixVecs<DimVec<double> >& coords);

    /// Destructor.
    ~SurfaceQuadrature()
    {}

    /// Adapts SurfaceQuadrature to \ref coords.
    void scaleSurfaceQuadrature(VectorOfFixVecs<DimVec<double> > &coords);

  protected:
    /// Pointer to the original quadrature
    Quadrature *quad;

    VectorOfFixVecs<DimVec<double> > coords;
  };

}

#endif // AMDIS_SURFACEQAUDRATURE_H
